#version 430
// vertex attributes 
layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;
layout(location = 3) in vec3 tangent;
layout(location = 4) in vec3 bitangent; // maybe not necessary

layout(location = 0) uniform mat4 view_proj_mat;
layout(location = 4) uniform mat4 model_mat;
layout(location = 8) uniform mat3 normal_mat;
layout(location = 16) uniform mat4 light_space_matrix;

// inputs for fragment shader
out vec3 vert_world_position;
out vec3 vert_normal;
out vec2 vert_uv;
out vec4 vert_light_pos;
out mat3 TBN;

void main()
{
  vec4 world_pos = model_mat * vec4(position, 1.0f);
  vert_world_position = vec3(world_pos);
  gl_Position = view_proj_mat * world_pos;

  vert_uv = uv;
  
  vert_normal = normal_mat * normal;
  
  vert_light_pos = light_space_matrix * world_pos;
    
  vec3 T = normalize(vec3(model_mat * vec4(tangent, 0.0f)));
  vec3 N = normalize(vec3(model_mat * vec4(normal, 0.0f)));
  vec3 B = normalize(vec3(model_mat * vec4(bitangent, 0.0f)));
  T = normalize(T - dot(T, N) * N);

  TBN = mat3(T, B, N);
}