#version 430

in vec2 tex_coord;

layout(location = 0) out vec4 frag_color;

layout(location = 0) uniform sampler2D frame_tex;
layout(location = 1) uniform sampler2D bloom_tex;
layout(location = 2) uniform sampler3D lut_tex;
layout(location = 22) uniform sampler3D lut_tex_2;
layout(location = 3) uniform vec3 lut_scale;
layout(location = 4) uniform vec3 lut_offset;
layout(location = 5) uniform vec3 gammas;
layout(location = 7) uniform sampler2D disort_tex;
layout(location = 8) uniform sampler2D screen_blend_tex;

layout(location = 38) uniform sampler2D noise_tex;
layout(location = 36) uniform sampler2D depth_tex;

layout(location = 39) uniform float time;
layout(location = 41) uniform float mix_value;
layout(location = 43) uniform vec4 disort_vals;
layout(location = 44) uniform vec4 time_vals;

const float bloom_iterations = 2.0;

void main(void) {
    vec3 bloom = texture(bloom_tex, tex_coord).rgb * bloom_iterations;
    
    
    float disorter = texture(disort_tex, tex_coord).r;
    float noise_shift = texture(noise_tex, tex_coord+vec2(time*time_vals.g)).r;
    
    float noise_wobble = sin(tex_coord.y*(disort_vals.x+noise_shift*disort_vals.z)+time*time_vals.r)*(disort_vals.y+noise_shift*disort_vals.w); 
    vec2 disort = disorter*vec2(noise_wobble,0);
    
    vec2 disorted_coord = tex_coord+disort;
    vec3 raw_color = texture(frame_tex, disorted_coord).rgb;
    if (disorted_coord.x < 0 || disorted_coord.x > 1 || disorted_coord.y < 0 || disorted_coord.y > 1) {
      raw_color = vec3(0);
    }
    vec4 screen_blend = texture(screen_blend_tex, vec2(tex_coord.x, -tex_coord.y));
    
    //https://developer.nvidia.com/gpugems/GPUGems2/gpugems2_chapter24.html
    vec4 lut_1 = texture(lut_tex, lut_scale * raw_color + lut_offset);
    vec4 lut_2 = texture(lut_tex_2, lut_scale * raw_color + lut_offset);
    
    float zNear = 0.1;    // TODO: Replace by the zNear of your perspective projections
    float zFar  = 1000.0; // TODO: Replace by the zFar  of your perspective projection
    float depth = texture(depth_tex, tex_coord).x;
    float corrected = (2.0 * zNear) / (zFar + zNear - depth * (zFar - zNear));
    
    //frag_color = vec4(vec3(corrected), 1.0);
    frag_color = screen_blend*mix(lut_1, lut_2, mix_value);
}