#version 430

in vec2 tex_coord;

layout(location = 0) out vec4 frag_color;

layout(location = 0) uniform sampler2D frame_tex;
layout(location = 1) uniform sampler2D bloom_tex;
layout(location = 2) uniform sampler3D lut_tex;
layout(location = 22) uniform sampler3D lut_tex_2;
layout(location = 3) uniform vec3 lut_scale;
layout(location = 4) uniform vec3 lut_offset;
layout(location = 5) uniform vec3 gammas;
layout(location = 7) uniform sampler2D disort_tex;
layout(location = 8) uniform sampler2D screen_blend_tex;

layout(location = 38) uniform sampler2D noise_tex;
layout(location = 36) uniform sampler2D depth_tex;

layout(location = 39) uniform float time;
layout(location = 41) uniform float mix_value;
const float disort_max = 0.2;

const float bloom_iterations = 2.0;

void main(void) {
    vec3 bloom = texture(bloom_tex, tex_coord).rgb * bloom_iterations;
    
    //https://developer.nvidia.com/gpugems/GPUGems2/gpugems2_chapter24.html
    
    //frag_color = vec4(pow(texture(frame_tex, tex_coord).rgb + bloom, gammas), 1.0);
    float frequency = 50.0;
    float amplitude = 0.002;
    float disorter = texture(disort_tex, tex_coord).r;
    float noise_shift = texture(noise_tex, tex_coord+vec2(time*100)).r;
    
    vec2 disort = disorter*vec2(sin(tex_coord.y*(frequency+noise_shift*20)+time*10000)*(amplitude+noise_shift*0.05),0);
    //disort.x = disorter*noise_shift;
    vec3 raw_color = texture(frame_tex, tex_coord+disort).rgb;
    
    vec4 screen_blend = texture(screen_blend_tex, vec2(tex_coord.x, -tex_coord.y));
    
    vec4 lut_1 = texture(lut_tex, lut_scale * raw_color + lut_offset);
    vec4 lut_2 = texture(lut_tex_2, lut_scale * raw_color + lut_offset);
    
    float zNear = 0.1;    // TODO: Replace by the zNear of your perspective projections
    float zFar  = 1000.0; // TODO: Replace by the zFar  of your perspective projection
    float depth = texture(depth_tex, tex_coord).x;
    float corrected = (2.0 * zNear) / (zFar + zNear - depth * (zFar - zNear));
    
    //frag_color = vec4(vec3(corrected), 1.0);
    frag_color = screen_blend*mix(lut_1, lut_2, mix_value);
}